@@ -1,15 +1,14 @@
 package Lyrics::Fetcher::LyricWiki;
 
-# $Id: LyricWiki.pm 724 2009-09-08 21:51:59Z davidp $
+# $Id: LyricWiki.pm 768 2009-10-18 16:50:48Z davidp $
 
 use 5.005000;
 use strict;
 use warnings;
 use LWP::UserAgent;
-use HTML::TagParser;
 use Carp;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 # the HTTP User-Agent we'll send:
 our $AGENT = "Perl/Lyrics::Fetcher::LyricWiki $VERSION";
@@ -63,12 +62,17 @@ sub fetch {
     my $ua = LWP::UserAgent->new();
     $ua->agent($AGENT);
 
+    # We'll fetch the edit page for the lyrics, as it provides nice clean text
+    # to parse out, without ringtone adverts etc which made the HTML
+    # unparseable.
     my $url = join ':', map { s/\s+/_/; $_ } ($artist, $song);
-    my $resp = $ua->get("http://lyrics.wikia.com/lyrics/$url");
+    my $resp = $ua->get("http://lyrics.wikia.com/index.php?action=edit"
+        . "&title=$url");
     
     if (!$resp->is_success) {
         if ($resp->status_line =~ /404/) {
-            # Lyrics for this song not found
+            # Lyrics for this song not found (this doesn't seem to happen, we
+            # get a 200 anyway, handled below...)
             $Lyrics::Fetcher::Error = 'Lyrics not found';
             return;
         } else {
@@ -78,21 +82,33 @@ sub fetch {
         }
     }
 
+    # Check it wasn't the "page doesn't exist yet" error page
+    if ($resp->content =~ /a link to a page that doesn't exist yet/) {
+        $Lyrics::Fetcher::Error = 'Lyrics not found';
+        return;
+    }
+   
+    # If it was a redirect, we should follow it; just call ourselves again.
+    # TODO: make sure we don't end up with infinite recursion if there's a
+    # redirect loop.
+    if (my($newartist, $newtitle) = 
+        $resp->content =~ m{\#REDIRECT \s+ \[\[ ([^:]+) : ([^:]+) \]\] }xi)
+    {
+        return __PACKAGE__->fetch($newartist, $newtitle);
+    }
+
     # OK, parse the HTML:
     my $html = $resp->content;
-    $html =~ s{<br\s+/?>}{%newline%}gi;
-    my $parser = HTML::TagParser->new( $html );
-
-    if (my $lyricsdiv = $parser->getElementsByClassName('lyricbox')) {
+    my ($lyrics) = $html =~ m{ &lt;lyrics?&gt; (.+?) &lt;/lyrics?&gt;}xms;
+    
+    if ($lyrics) {
+        # Looks like we got something usable:
         $Lyrics::Fetcher::Error = 'OK';
-        my $lyrics = $lyricsdiv->innerText;
-        $lyrics =~ s/%newline%/\n/g;
         return $lyrics;
     } else {
         $Lyrics::Fetcher::Error = 'No lyrics parsed from page';
         return;
     }
-
 }
 
 
@@ -142,10 +158,5 @@ This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.7 or,
 at your option, any later version of Perl 5 you may have available.
 
-Legal disclaimer: I have no connection with the owners of www.azlyrics.com.
-Lyrics fetched by this script may be copyrighted by the authors, it's up to 
-you to determine whether this is the case, and if so, whether you are entitled 
-to request/use those lyrics.  You will almost certainly not be allowed to use
-the lyrics obtained for any commercial purposes.
 
 =cut
